MAJOR = 0
MINOR = 0
PATCH = 1
SUFFIX = ""

NAME = "sqs_py"

AUTHOR = MAINTAINER = {
    "name": "wengjiyao",
    "email": "w.j@live.ca",
}

AUTHORS = MAINTAINERS = (
    f"{AUTHOR.get('name')} <{AUTHOR.get('email')}>",
)

__version__ = VERSION = f"{MAJOR}.{MINOR}.{PATCH}{SUFFIX}"
